---------------------------------------------------------------------------
--
-- UpdateMetricsValues
--
-- Update values to Metrics from #MetricsData temporary table.
-- Values are loaded to Metrics in batches. The number of lines
-- contained in each batch is configurable by script user; the acceptable
-- values are between one (1) and one thousand (1000).
--
-- After each batch, this script is called recursively using the --#Run command.
--
-- Configuration of batch size: set the batch size (1-1000) at line 103 of the script.
--			Default value for batch size is 1.
--
-- Input: 	#_Arg1 			Contains [CurrentDataRow] column that maintains
--							the number of rows processed so far.
--			#SessionData 	MEA web service session identifier.
-- 			#ModelId		Id of the target metrics model (SC.12345)
--			#NumberOfFailures Contains the number of failures contained so far.
--							If this temporary table doesn't exists, it is created
--							at the beginning. The value is increased when encountering
--							an error.
--			#MetricsData	A temporary table containing the data to be loaded.
--							The table must contain the following columns (with types)
--				* Id (Int, Not null, Identity(1,1)): contains a unique identity for the data row.
--				* Scorecard (VARCHAR(80)): Identifier of the scorecard holding the value.
--				* Element (VARCHAR(80)): Identifier of the element holding the value.
--				* Series (VARCHAR(3)): Identifier of the series holding the value.
--				* Pediod (VARCHAR(20)): Period declaration in form e.g. XML-formatted date or quarter declaration.
--				* Value (VARCHAR(20)): The value to be set.
--
--				SQL create declaration for the #MetricsData:
-- 				CREATE TABLE #MetricsData
--				(
--					Id Int NOT NULL IDENTITY(1,1),
--					Scorecard VARCHAR(80),
--					Element VARCHAR(80),
--					Series VARCHAR(3),
--					Period VARCHAR(20),
--					Value VARCHAR(20)
--				);
--
-- Request JSON example:
-- {"sessionId":"0wLVHZm9yqKxETQ4sE-Fpg","model":"Requirements Management Demo","values":[["DEV_EFFORT","MO1340463212_1","DONE_EFFORT","2017-01-24","1000000"],["DEV_EFFORT","MO1340463212_1","DONE_EFFORT","2017-01-25","1000000"],["DEV_EFFORT","MO1340463212_1","DONE_EFFORT","2017-01-26","1000000"],["DEV_EFFORT","MO1340463212_1","DONE_EFFORT","2017-01-27","1000000"],["DEV_EFFORT","MO1340463212_1","DONE_EFFORT","2017-02-28","1000000"]],"options":""}
---------------------------------------------------------------------------

-- This script can be only called from another script.
(SELECT 'MessageText', 'Temporary table #ModelId must be set before calling UpdateMetricsValues.') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#ModelId') IS NOT NULL,'0','1'))
--#Exit 

-- SessionData must exists.
(SELECT 'MessageText', 'Can not find session data (#SessionData temporary table not set)!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#SessionData') IS NOT NULL,'0','1'))
--#Exit 

-- MetricsData must exists.
(SELECT 'MessageText', 'Can not find Metrics data (#MetricsData temporary table not set)!') UNION ALL
(SELECT 'Exit', IIF(OBJECT_ID('tempdb..#MetricsData') IS NOT NULL,'0','1'))
--#Exit 

-- Create a temporary table holding number of failures, if it doesn't exist yet.
IF OBJECT_ID('tempdb..#NumberOfFailures') IS NULL
	SELECT 0 AS VAL INTO #NumberOfFailures;

	-- Exit script if no elements are left.
(SELECT 'MessageText', 'No Metrics data left') UNION ALL
(SELECT 'Exit', IIF((SELECT COUNT(*) FROM #MetricsData) > 0,'0','1'))
--#Exit

-- Create #MetricsRow temporary table conforming with #MetricsData structure
IF OBJECT_ID('tempdb..#MetricsRow') IS NULL
BEGIN
	SELECT TOP 1 *
	INTO #MetricsRow
	FROM #MetricsData m
	ORDER BY Id
END

DELETE FROM #MetricsRow

GO;

---------------------------------------------------------------------------
---- UPDATE SC VALUES
---------------------------------------------------------------------------

DECLARE @SessionString AS NVARCHAR(MAX);
DECLARE @Body AS NVARCHAR(MAX);
DECLARE @ArrVals AS NVARCHAR(MAX);

SELECT @SessionString = [SES_ID] FROM #SessionData;

---------------------------------------------------------------------------
---- SEND ONE ROW TO METRICS
---------------------------------------------------------------------------

print 'UpdateMetricsValues: SessionString: [' + @SessionString + ']'

-- Clear #MetricsRow
DELETE FROM #MetricsRow

------------------------------------------------------------
------------------------------------------------------------
-- UPDATE THE BATCH SIZE TO THE SELECT CLAUSE BELOW (by default 1)
-- Read a batch of data from #MetricsData to #MetricsRow
------------------------------------------------------------
------------------------------------------------------------
SET IDENTITY_INSERT #MetricsRow ON
INSERT INTO #MetricsRow([Id], [Scorecard], [Element], [Series], [Period], [Value])
SELECT TOP 1 [Id], [Scorecard], [Element], [Series], [Period], [Value] 
FROM #MetricsData
ORDER BY Id
SET IDENTITY_INSERT #MetricsRow OFF

-- Remove the row read from #MetricsData
DELETE FROM #MetricsData WHERE Id IN (SELECT Id FROM #MetricsRow)

-- SELECT STUFF((SELECT ',' + Txt 
--            FROM ConcatenationDemo
--            FOR XML PATH('')) ,1,1,'') AS Txt
			
SET @ArrVals =  STUFF((SELECT ',' + 
 '["' + CONVERT(NVARCHAR,[Element]) + '","' + 
        CONVERT(NVARCHAR,[Scorecard]) +'","' +
        CONVERT(NVARCHAR,[Series]) +'","' +
        CONVERT(NVARCHAR,[Period]) + '","' +
        CONVERT(NVARCHAR,[Value]) +'"]'
	FROM #MetricsRow
	FOR XML PATH('')) ,1,1,'')

SELECT @Body = '{"sessionId":"' + @SessionString + '","model":"' + (SELECT VAL FROM #MetricsModelName) + '","values":['
+ @ArrVals + '],"options":""}'
FROM #MetricsRow

print 'UpdateMetricsValues: Body: [' + @Body + ']'

(SELECT 'Method', 'POST') UNION ALL
(SELECT 'Address', (SELECT VAL FROM #BaseUrl) + '/SetSCMeasureValuesForModel') UNION ALL
(SELECT 'Body', @Body) UNION ALL
(SELECT 'ContentType', 'application/json') UNION ALL
(SELECT 'ExecuteInClientSide', '1') UNION ALL
(SELECT 'Accept', '*/*')
--#CallWebService

-- Get current data row
DECLARE @CurrentDataRow AS INT;
IF OBJECT_ID('tempdb..#_Arg1') IS NOT NULL
 SELECT @CurrentDataRow = CONVERT(INT, [CurrentDataRow]) FROM #_Arg1;
ELSE
 SET @CurrentDataRow = 1;

-- Log an error message, if web service call didn't succeed.
IF CONVERT(INT, @_ResponseSuccess) = 1
 GOTO SetSCMeasureSuccess
ELSE
 BEGIN
  print 'Update of scorecard values failed at batch row ' + CONVERT(VARCHAR(10), @CurrentDataRow) + ' : ' + @_ResponseText;
  UPDATE #NumberOfFailures SET VAL = VAL + 1; 
 END
SetScMeasureSuccess:

-- Get number of failures so far
DECLARE @NumberOfFailures AS INT = (SELECT CONVERT(INT, [VAL]) FROM #NumberOfFailures);

(SELECT 'MessageText', 'Tried to update ' + CONVERT(VARCHAR, (@CurrentDataRow))
+ ' batches to Metrics model "'
+ (SELECT VAL FROM #MetricsModelName) + '". Number of failures: ' + CONVERT(VARCHAR, (@NumberOfFailures))) UNION ALL
(SELECT 'Exit', IIF(CONVERT(INT,COUNT(*)) > 0,'0','1') FROM #MetricsData )
--#Exit 

-- Recursively call this script again.
DECLARE @CurrentDataRow AS INT;
IF OBJECT_ID('tempdb..#_Arg1') IS NOT NULL
 SELECT @CurrentDataRow = CONVERT(INT, [CurrentDataRow]) FROM #_Arg1;
ELSE
 SET @CurrentDataRow = 1;

 print 'UpdateMetricsValues: Current data row: ' + CONVERT(NVARCHAR, @CurrentDataRow)

 SET @CurrentDataRow = @CurrentDataRow + 1;

(SELECT 'ScriptId', @_CurrentScriptId)
(SELECT @CurrentDataRow AS CurrentDataRow)
--#Run
